#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
 @file pcd_to_pointcloud.py
 @brief PointCloud Publisher
 @date $Date$

 @author East Japan Accounting Center Co., Ltd.

 (c)2020 The University of Aizu
 This software is released under the New BSD License.
"""

import sys
import threading

import rospy

from std_msgs.msg import Header
from sensor_msgs.msg import PointCloud2, PointField
import sensor_msgs.point_cloud2 as pc2

import pcl

HEADER = Header(frame_id='/base_link')

FIELDS = [
    PointField(name='x', offset=0, datatype=PointField.FLOAT32, count=1),
    PointField(name='y', offset=4, datatype=PointField.FLOAT32, count=1),
    PointField(name='z', offset=8, datatype=PointField.FLOAT32, count=1),
    PointField(name='intensity', offset=12, datatype=PointField.FLOAT32, count=1),
]


class Pcd_to_Pointcloud(object):

    mutex = threading.Semaphore(1)

    def __init__(self):
        rospy.init_node('pcd_to_pointcloud')
        rospy.on_shutdown(self.shutdown)

        self.pc2_publisher = rospy.Publisher('/cloud_pcd', PointCloud2, queue_size=1)

    def publish_points(self, txtfile):
        """
        Publish PointCloud

        Parameters
        ----------
        txtfile: string
            PCD filename list
        """

        pcd_file_list = []
        cnt = 0
        with open(txtfile, "r") as f:
            pcd_file_list = f.read().split()

        r = rospy.Rate(1)
        list_len = len(pcd_file_list)
        while cnt != list_len and not rospy.is_shutdown():
            self.mutex.acquire()

            pcd_load = pcl.load_XYZI(pcd_file_list[cnt])
            np_cloud = pcd_load.to_array()
            point_cloud = pc2.create_cloud(HEADER, FIELDS, np_cloud)
            self.pc2_publisher.publish(point_cloud)
            rospy.loginfo("Publish Pcd2Pointcloud %s of %s", cnt, list_len)

            self.mutex.release()
            r.sleep()

            cnt += 1

    def shutdown(self):
        """
        Unregister Publisher
        """

        try:
            self.pc2_publisher.unregister()
        except rospy.ROSException:
            pass


if __name__ == '__main__':

    try:
        if len(sys.argv) == 2:
            node = Pcd_to_Pointcloud()
            node.publish_points(sys.argv[1])
        else:
            import pcd_to_pointcloud
            help(pcd_to_pointcloud)

    except rospy.ROSInterruptException:
        pass
